(***** Scanner *****)

type token =
   Tok_Num of char
 | Tok_Sum
 | Tok_END

(* OCaml regexps *)
     
#load "str.cma"

let re_num = Str.regexp "[0-9]" 	(* single digit number *)
let re_add = Str.regexp "+"

exception IllegalExpression of string

(*
  tokenize : string -> token list
  converts string into a list of tokens (i.e., terminals)
*)
let tokenize str =
 let rec tok pos s =
   if pos >= String.length s then
     [Tok_END]
   else 
     if (Str.string_match re_num s pos) then
       let token = Str.matched_string s in
         (Tok_Num token.[0])::(tok (pos+1) s)
     else if (Str.string_match re_add s pos) then
       Tok_Sum::(tok (pos+1) s)
     else
       raise (IllegalExpression "tokenize")
 in
 tok 0 str
 
(*
  tok_to_str : token -> string
  converts token into a string 
*)
let tok_to_str t =
  match t with
      Tok_Num v -> (Char.escaped v)
    | Tok_Sum -> "+"
    | Tok_END -> "END"

(****** FROM CLASS ******)

(*
type token = string

let tokenize s =
  let l = String.length s in
  let rec tok sidx slen =
    if sidx >= l then ("",sidx)
    else if String.get s sidx = ' ' then
      tok (sidx+1) 1
    else if (sidx+slen) >= l then
      (String.sub s sidx slen,l)
    else if String.get s (sidx+slen) = ' ' then
      (String.sub s sidx slen, sidx+slen)
    else
      tok sidx (slen+1) in
  let rec alltoks idx =
    let (t,idx') = tok idx 1 in
    if t = "" then []
    else t::alltoks idx' in
  alltoks 0
;;

tokenize "this is a string with spaces ...";;
*)
